@@ -256,7 +256,7 @@ class MessagingClient:
         limit: int = 50, 
         offset: int = 0,
         provider: str = "openai"
-    ) -> Optional[List[Dict[str, str]]]:
+    ) -> Optional[List[Dict[str, Any]]]:
         """
         Get message history from a conversation formatted for the specified LLM provider.
         
@@ -288,7 +288,12 @@ class MessagingClient:
                 sorted_messages = sorted(history.messages, key=lambda x: x.get('timestamp', 0))
                 
                 if provider == "openai":
-                    return self._format_for_openai(sorted_messages)
+                    # First format with the standard method
+                    formatted = self._format_for_openai(sorted_messages)
+                    
+                    # Apply additional formatting for already-formatted messages
+                    # This handles the case where backend returns role/content instead of raw data
+                    return self._post_process_formatted_messages(formatted)
                 elif provider == "anthropic":
                     # TODO: Implement Anthropic Claude format
                     # return self._format_for_anthropic(sorted_messages)
@@ -308,19 +313,79 @@ class MessagingClient:
             logger.error(f"Failed to get conversation history: {e}")
             return None
     
-    def _format_for_openai(self, messages: List[Dict[str, Any]]) -> List[Dict[str, str]]:
-        """Format messages for OpenAI chat completion API."""
+    def _format_for_openai(self, messages: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
+        """Format messages for OpenAI chat completion API, including multimodal content."""
         formatted_messages = []
         
         for msg in messages:
             sender_type = msg.get('senderType', '')
             content = msg.get('content', '')
+            message_type = msg.get('messageType', 'text')
+            metadata = msg.get('metadata', {})
             
-            if sender_type == 'user' and content:
-                formatted_messages.append({
-                    "role": "user",
-                    "content": content
-                })
+            if sender_type == 'user':
+                # Check if this is a file message with an image
+                if message_type == 'file' and metadata:
+                    file_info = metadata.get('file', {})
+                    mime_type = file_info.get('mimeType', '')
+                    
+                    # Handle image files
+                    if mime_type and mime_type.startswith('image/'):
+                        # Create multimodal content with image
+                        image_url = file_info.get('url', '')
+                        logger.debug(f"Processing image message: mime_type={mime_type}, has_url={bool(image_url)}")
+                        if image_url:
+                            logger.debug(f"Creating multimodal message with image URL: {image_url}")
+                            formatted_msg = {
+                                "role": "user",
+                                "content": [
+                                    {"type": "text", "text": content or "What's in this image?"},
+                                    {
+                                        "type": "image_url",
+                                        "image_url": {"url": image_url}
+                                    }
+                                ]
+                            }
+                            formatted_messages.append(formatted_msg)
+                        else:
+                            # Fallback to text if no URL
+                            logger.debug("No image URL found, falling back to text content")
+                            formatted_messages.append({
+                                "role": "user",
+                                "content": content
+                            })
+                    else:
+                        # Non-image files, just use text content
+                        formatted_messages.append({
+                            "role": "user",
+                            "content": content
+                        })
+                elif content:
+                    # Workaround: Check if content looks like an image URL
+                    if isinstance(content, str) and (
+                        content.startswith('http') and 
+                        any(ext in content.lower() for ext in ['.png', '.jpg', '.jpeg', '.gif', '.webp', '.bmp'])
+                    ):
+                        # This is likely an image URL sent as plain text
+                        logger.debug(f"Detected image URL in plain text message: {content}")
+                        formatted_msg = {
+                            "role": "user",
+                            "content": [
+                                {"type": "text", "text": "Analyze this image:"},
+                                {
+                                    "type": "image_url",
+                                    "image_url": {"url": content}
+                                }
+                            ]
+                        }
+                        formatted_messages.append(formatted_msg)
+                    else:
+                        # Regular text message
+                        formatted_messages.append({
+                            "role": "user",
+                            "content": content
+                        })
+                    
             elif sender_type == 'agent' and content:
                 formatted_messages.append({
                     "role": "assistant", 
@@ -330,6 +395,40 @@ class MessagingClient:
         
         return formatted_messages
     
+    def _post_process_formatted_messages(self, messages: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
+        """Post-process already formatted messages to handle image URLs in content."""
+        processed_messages = []
+        
+        for msg in messages:
+            role = msg.get('role', '')
+            content = msg.get('content', '')
+            
+            # Only process user messages with string content
+            if role == 'user' and isinstance(content, str) and content:
+                # Check if content looks like an image URL
+                if content.startswith('http') and any(ext in content.lower() for ext in ['.png', '.jpg', '.jpeg', '.gif', '.webp', '.bmp']):
+                    # Convert to multimodal format
+                    logger.debug(f"Post-processing: Converting plain text image URL to multimodal format: {content}")
+                    processed_msg = {
+                        "role": "user",
+                        "content": [
+                            {"type": "text", "text": "Analyze this image:"},
+                            {
+                                "type": "image_url",
+                                "image_url": {"url": content}
+                            }
+                        ]
+                    }
+                    processed_messages.append(processed_msg)
+                else:
+                    # Keep as-is
+                    processed_messages.append(msg)
+            else:
+                # Keep non-user messages and already-formatted content as-is
+                processed_messages.append(msg)
+        
+        return processed_messages
+    
     async def register_agent(self, endpoint_url: str) -> bool:
         """
         Register agent with the messaging system.
