@@ -20,29 +20,33 @@ logger = logging.getLogger(__name__)
 class OpenAIProvider(Provider):
     """OpenAI LLM provider implementation."""
     
-    def __init__(self):
-        """Initialize OpenAI client."""
+    def __init__(self, billing_mode: str = "builder"):
+        """
+        Initialize OpenAI client.
+        
+        Args:
+            billing_mode: "builder" (use env DEMIURG_USER_ID) or "user" (use dynamic user_id)
+        """
+        self.billing_mode = billing_mode
+        self.base_url = os.getenv('OPENAI_BASE_URL', 'http://demiurg-openai-proxy:3001/v1')
+        self._current_user_id = None  # For dynamic user mode
+        
         try:
-            # Use the OpenAI proxy service
-            base_url = os.getenv('OPENAI_BASE_URL', 'http://demiurg-openai-proxy:3001/v1')
+            # Initialize clients without headers (will be set per request in user mode)
+            if billing_mode == "builder":
+                # Builder mode: use fixed headers
+                default_headers = {
+                    'X-Session-Id': os.getenv('DEMIURG_SESSION_ID'),
+                    'X-Agent-Id': os.getenv('DEMIURG_AGENT_ID'),
+                    'X-User-Id': os.getenv('DEMIURG_USER_ID'),
+                }
+                self.client = OpenAI(base_url=self.base_url, default_headers=default_headers)
+                self.async_client = AsyncOpenAI(base_url=self.base_url, default_headers=default_headers)
+            else:
+                # User mode: no default headers, will be set per request
+                self.client = OpenAI(base_url=self.base_url)
+                self.async_client = AsyncOpenAI(base_url=self.base_url)
             
-            # Initialize both sync and async clients
-            self.client = OpenAI(
-                base_url=base_url,
-                default_headers={
-                    'X-Session-Id': os.getenv('DEMIURG_SESSION_ID'),
-                    'X-Agent-Id': os.getenv('DEMIURG_AGENT_ID'),
-                    'X-User-Id': os.getenv('DEMIURG_USER_ID'),
-                }
-            )
-            self.async_client = AsyncOpenAI(
-                base_url=base_url,
-                default_headers={
-                    'X-Session-Id': os.getenv('DEMIURG_SESSION_ID'),
-                    'X-Agent-Id': os.getenv('DEMIURG_AGENT_ID'),
-                    'X-User-Id': os.getenv('DEMIURG_USER_ID'),
-                }
-            )
             self.available = True
         except Exception as e:
             logger.warning(f"OpenAI client initialization failed: {e}")
@@ -50,6 +54,24 @@ class OpenAIProvider(Provider):
             self.async_client = None
             self.available = False
     
+    def set_current_user(self, user_id: Optional[str]):
+        """Set the current user ID for dynamic billing mode."""
+        self._current_user_id = user_id
+    
+    def _get_headers(self) -> Dict[str, str]:
+        """Get headers based on billing mode and current user."""
+        headers = {
+            'X-Session-Id': os.getenv('DEMIURG_SESSION_ID', ''),
+            'X-Agent-Id': os.getenv('DEMIURG_AGENT_ID', ''),
+        }
+        
+        if self.billing_mode == "builder":
+            headers['X-User-Id'] = os.getenv('DEMIURG_USER_ID', '')
+        elif self.billing_mode == "user" and self._current_user_id:
+            headers['X-User-Id'] = self._current_user_id
+        
+        return headers
+    
     async def process(
         self,
         messages: List[Dict[str, Any]],
@@ -86,6 +108,38 @@ class OpenAIProvider(Provider):
             # Log request details
             logger.debug(f"Processing with model: {model}, temp: {temperature}, max_tokens: {max_tokens}")
             
+            # Debug: Log the entire conversation being sent to OpenAI
+            logger.debug("=== FULL CONVERSATION SENT TO OPENAI ===")
+            logger.debug(f"Number of messages: {len(messages)}")
+            for i, msg in enumerate(messages):
+                logger.debug(f"Message {i+1}:")
+                logger.debug(f"  Role: {msg.get('role', 'unknown')}")
+                if isinstance(msg.get('content'), str):
+                    logger.debug(f"  Content: {msg.get('content', '')[:200]}{'...' if len(msg.get('content', '')) > 200 else ''}")
+                elif isinstance(msg.get('content'), list):
+                    logger.debug(f"  Content (multimodal): {len(msg.get('content', []))} parts")
+                    for j, part in enumerate(msg.get('content', [])):
+                        if part.get('type') == 'text':
+                            logger.debug(f"    Part {j+1} (text): {part.get('text', '')[:100]}{'...' if len(part.get('text', '')) > 100 else ''}")
+                        elif part.get('type') == 'image_url':
+                            image_url = part.get('image_url', {}).get('url', '')
+                            if image_url.startswith('data:'):
+                                logger.debug(f"    Part {j+1} (image): base64 data ({len(image_url)} chars)")
+                            else:
+                                logger.debug(f"    Part {j+1} (image): {image_url}")
+                        else:
+                            logger.debug(f"    Part {j+1} ({part.get('type', 'unknown')}): {str(part)[:100]}...")
+                if msg.get('tool_calls'):
+                    logger.debug(f"  Tool calls: {len(msg.get('tool_calls', []))} calls")
+            logger.debug("=== END OF CONVERSATION ===")
+            
+            # Debug: Log full message structure as JSON for easier analysis
+            try:
+                logger.debug("Full messages structure (JSON):")
+                logger.debug(json.dumps(messages, indent=2, default=str))
+            except Exception as e:
+                logger.debug(f"Could not serialize messages to JSON: {e}")
+            
             # Prepare request
             request_params = {
                 "model": model,
@@ -98,8 +152,15 @@ class OpenAIProvider(Provider):
             if tools:
                 request_params["tools"] = tools
             
-            # Make API call
-            completion = await self.async_client.chat.completions.create(**request_params)
+            # Make API call with dynamic headers if in user mode
+            if self.billing_mode == "user":
+                # Need to create a new client with updated headers for this request
+                headers = self._get_headers()
+                temp_client = AsyncOpenAI(base_url=self.base_url, default_headers=headers)
+                completion = await temp_client.chat.completions.create(**request_params)
+            else:
+                # Use the default client in builder mode
+                completion = await self.async_client.chat.completions.create(**request_params)
             
             # Handle response
             response_message = completion.choices[0].message
@@ -178,12 +239,22 @@ class OpenAIProvider(Provider):
         
         try:
             with open(file_path, 'rb') as audio_file:
-                transcription = await self.async_client.audio.transcriptions.create(
-                    model=model,
-                    file=audio_file,
-                    response_format=kwargs.get('response_format', 'text'),
-                    **kwargs
-                )
+                if self.billing_mode == "user":
+                    headers = self._get_headers()
+                    temp_client = AsyncOpenAI(base_url=self.base_url, default_headers=headers)
+                    transcription = await temp_client.audio.transcriptions.create(
+                        model=model,
+                        file=audio_file,
+                        response_format=kwargs.get('response_format', 'text'),
+                        **kwargs
+                    )
+                else:
+                    transcription = await self.async_client.audio.transcriptions.create(
+                        model=model,
+                        file=audio_file,
+                        response_format=kwargs.get('response_format', 'text'),
+                        **kwargs
+                    )
                 return transcription
         except Exception as e:
             logger.error(f"Error transcribing audio: {e}")
@@ -227,13 +298,24 @@ class OpenAIProvider(Provider):
             raise ProviderError("OpenAI client is not available")
         
         try:
-            response = await self.async_client.audio.speech.create(
-                model=model,
-                voice=voice,
-                input=text,
-                response_format=response_format,
-                **kwargs
-            )
+            if self.billing_mode == "user":
+                headers = self._get_headers()
+                temp_client = AsyncOpenAI(base_url=self.base_url, default_headers=headers)
+                response = await temp_client.audio.speech.create(
+                    model=model,
+                    voice=voice,
+                    input=text,
+                    response_format=response_format,
+                    **kwargs
+                )
+            else:
+                response = await self.async_client.audio.speech.create(
+                    model=model,
+                    voice=voice,
+                    input=text,
+                    response_format=response_format,
+                    **kwargs
+                )
             
             # For async client, we need to use aread() to get the bytes
             audio_data = await response.aread()
@@ -297,7 +379,12 @@ class OpenAIProvider(Provider):
                 params["response_format"] = kwargs.get("response_format", "b64_json")
             # gpt-image-1 doesn't need size/quality/style
             
-            response = await self.async_client.images.generate(**params)
+            if self.billing_mode == "user":
+                headers = self._get_headers()
+                temp_client = AsyncOpenAI(base_url=self.base_url, default_headers=headers)
+                response = await temp_client.images.generate(**params)
+            else:
+                response = await self.async_client.images.generate(**params)
             
             # Format response
             images = []
